@@ -244,10 +244,9 @@ def generate_client_id(length=30, chars=CLIENT_ID_CHARACTER_SET):
 
 def add_params_to_qs(query, params):
     """Extend a query with a list of two-tuples."""
-    if isinstance(query, dict):
-        queryparams = query.items()
-    else:
-        queryparams = urlparse.parse_qsl(query, keep_blank_values=True)
+    if isinstance(params, dict):
+        params = params.items()
+    queryparams = urlparse.parse_qsl(query, keep_blank_values=True)
     queryparams.extend(params)
     return urlencode(queryparams)
 
@@ -383,7 +382,7 @@ class Request(object):
     @property
     def duplicate_params(self):
         seen_keys = collections.defaultdict(int)
-        all_keys = (p[0] for p in self.decoded_body or [] + self.uri_query_params)
+        all_keys = (p[0] for p in (self.decoded_body or []) + self.uri_query_params)
         for k in all_keys:
             seen_keys[k] += 1
         return [k for k, c in seen_keys.items() if c > 1]
